import 'package:bolter_flutter/src/navigation/bolter_route.dart';
import 'package:hive/hive.dart';

part 'navigation_state.g.dart';

class NoRouteException implements Exception {}

T throwException<T>() => throw NoRouteException();

@HiveType(typeId: 223)
class Navigation {
  @HiveField(0)
  final List<BolterRoute<Object>> routes;
  //possible memory leak, but very rare case
  final _routesCache = <BolterRoute<Object>>[];

  Navigation(this.routes);

  void push<R>(BolterRoute<R> route) {
    _routesCache.clear();
    routes.add(route);
  }

  void pushReplacement<R>(BolterRoute<R> route) {
    pop();
    routes.add(route);
  }

  void pop() {
    if (routes.length > 1) {
      final poped = routes.removeLast();
      _routesCache.add(poped);
    }
  }

  T findRoute<T extends BolterRoute>() {
    return routes.firstWhere((element) => element is T, orElse: () {
      return _routesCache.firstWhere((element) => element is T);
    }) as T;
  }

  void popUntil<T extends BolterRoute>() {
    while (routes.last is! T && routes.isNotEmpty) {
      pop();
    }
  }

  void pushAndRemoveUntil<R>(BolterRoute<R> route) {
    popUntil<Null>();
    routes.add(route);
  }
}

@HiveType(typeId: 222)
class TabNavigation {
  @HiveField(0)
  String tab;

  TabNavigation(this.tab);
}
